/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.00
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void ADC_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
uint16_t adcaverage(uint16_t *val);
void delay(void);
void GPIO_Configure(void);
void ADC_Configure(void);
void ADC_InterruptRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec ;
const uint8_t menu[] =
"************************************************\n\r"
" ADC demo \n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
" Using ADC in interrupt mode \n\r"
"************************************************\n\r";

volatile uint32_t fflag;
uint16_t adcval[8];
ADC_CFG_Type AD_config;
/**********************************************************************
 * @brief		ADC_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void ADC_IRQHandler_IT(void)
{
	uint16_t status;
	status = HAL_ADC_GetStatus(ADC);

	if (status & ADC_STAT_END){
		fflag=1;
		HAL_ADC_ClearStatus(ADC);	
	}
}


/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/*********************************************************************//**
 * @brief		delay
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void delay(void) 
{
	uint32_t i;
	
	for (i = 0; i < 0x1000 ; i++) ;
}

/**********************************************************************
 * @brief		adcaverage
 * @param[in]	None
 * @return		None
 **********************************************************************/
uint16_t adcaverage(uint16_t *val)
{
	uint32_t i;
	uint32_t adcavg;
	
	adcavg=0;			
	for (i=0;i<8;i++){	
		adcavg+=val[i];
	}
	adcavg>>=3;
	
	return	(adcavg);
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	HAL_GPIO_ConfigOutput(PA, 0, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 0, FUNC3);
	HAL_GPIO_ConfigPullup(PA, 0, DISPUPD);
}

/**********************************************************************
 * @brief		ADC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_Configure(void)
{
  AD_config.RefSel = ADC_REF_VDD;   //0:ADC_REF_VDD ,1:ADC_REF_AVREF
	AD_config.TrgSel = ADC_TRIGGER_DISABLE;	
	AD_config.InClkDiv = 14;  
	if(HAL_ADC_Init(ADC, &AD_config) != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();
	}
	
	HAL_ADC_ConfigInterrupt(ADC, ENABLE);
  
	/* Enable Interrupt for ADC channel */
	NVIC_SetPriority(ADC_IRQn, 3);
	NVIC_EnableIRQ(ADC_IRQn);
}
	

/**********************************************************************
 * @brief		ADC_InterruptRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_InterruptRun(void)
{	
	uint32_t adcavg;
	
	HAL_ADC_ChannelSel(ADC, 0); // select ch

	fflag=0;			
	HAL_ADC_ClearStatus(ADC);
	HAL_ADC_Start(ADC); // start 	
	while(fflag==0){}			
	adcval[0]=HAL_ADC_GetData(ADC);


	adcavg=adcaverage(adcval);
	#ifdef _DEBUG_MSG
	_DBG("AN0="); 

	_DBH16(adcavg); _DBG(" ");
	_DBG("\n\r");
	#endif

}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*ADC Configure*/
	ADC_Configure();

	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
   /* Infinite loop */
		ADC_InterruptRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
