/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void CRC_Configure(void);
void CRC_CheckSumCalculationRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"************************************************\n\r"
" CRC demo \n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
"\t - CRC Checksum \n\r"
"\t @ CRC_INIT = 0, CRC_IN = 0x12345678 \n\r"
"************************************************\n\r";
uint32_t 					chk_in;


/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}



/**********************************************************************
 * @brief		CRC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void CRC_Configure(void)
{

  uint16_t 					chk_sum_init;	
	uint16_t 					chk_sum;
	uint32_t 					i;
	
	if(HAL_CRC_Init() != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();
	}
  // Checksum calculation by Core
	chk_sum_init = 0;			/* CRC_INIT Value*/
	chk_sum = chk_sum_init;
	chk_in = 0x12345678;	/* CRC_IN Value*/
	for (i=0; i<4; i++) {
		chk_sum += (uint16_t) (chk_in >> (24-i*8)) & 0x00FF;
	}

	#ifdef _DEBUG_MSG
	_DBG_(" ");
	_DBG("[Manual Calculation result] chk_sum = "); _DBH16(chk_sum); _DBG_(" "); 
	#endif
	
	
	/* Checksum calculation by CRC block*/
	HAL_CRC_SetUserInput(chk_sum_init);
	HAL_CRC_ConfigUserMode(CRC_CHECKSUM, CRC_CCITT, CRC_MSB);

	
}
	

/**********************************************************************
 * @brief		CRC_CheckSumCalculationRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void CRC_CheckSumCalculationRun(void)
{	
  uint16_t 					chk_sum;
	
	CRCRun();
	HAL_CRC_UserInput(chk_in);
	chk_sum = HAL_CRC_GetData();
	CRCStop();

  #ifdef _DEBUG_MSG
	_DBG("[CRC Block Calculation Result] chk_sum = "); _DBH16(chk_sum); _DBG_(" ");  
	#endif
		
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*CRC Configure*/
	CRC_Configure();
  
	/* Enable IRQ Interrupts */
	__enable_irq();
	
  /* Calculation  Result */
	CRC_CheckSumCalculationRun();
	
	/* Infinite loop */
	while(1)
	{
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
