/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private function prototypes ---------------------------------------*/
void tt_flash_iap_pgm(void);
void tt_flash_iap_erase(void);
void GPIO_Configure(void);
void FMC_Configure(void);
void FMC_WriteEraselookRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/


/**********************************************************************
 * @brief		tt_flash_iap_pgm
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	tt_flash_iap_pgm(void)
{
	int	i;
	unsigned char	fbuffer[512];
	
	for(i=0;i<512;i++) {
		fbuffer[i]= i&0xff;
	}
	__disable_irq();
	ProgramPage(0,0xFE00,512, fbuffer);
	__enable_irq();;
}

/**********************************************************************
 * @brief		tt_flash_iap_erase
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	tt_flash_iap_erase(void)
{
	__disable_irq();
	EraseSector(0, 0xFE00);
	__enable_irq();
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/* Test Pin setting PE0*/
	HAL_GPIO_ConfigOutput(PE, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 0, DISPUPD);	
	HAL_GPIO_ClearPin(PE, _BIT(0));

	/* Test Pin setting PE1*/
	HAL_GPIO_ConfigOutput(PE, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 1, DISPUPD);
	HAL_GPIO_ClearPin(PE, _BIT(1));

}

/**********************************************************************
 * @brief		FMC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void FMC_Configure(void)
{
	FMC->MR = 0x66;
	FMC->MR = 0x99;
	FMC->LOCK = 0x00000000;	/* flash read protection on*/
	FMC->MR = 0x00;
	FMC->MR = 0x00;	
	
	LockEraseChip();

	FMC->MR = 0x66;
	FMC->MR = 0x99;
	FMC->LOCK = 0x000000FF;	/* flash read protection off*/
	FMC->MR = 0x00;
	FMC->MR = 0x00;   
	
}
	

/**********************************************************************
 * @brief		FMC_WriteEraselookRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void FMC_WriteEraselookRun(void)
{	
		tt_flash_iap_erase();
		__NOP();	
		
		tt_flash_iap_pgm();
		__NOP();
		
		EraseChipBBlock();
		__NOP();
		
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*SysTick Configure*/
	FMC_Configure();

	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
   /* Infinite loop */
		FMC_WriteEraselookRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /*Disable  Interrupt*/
	__disable_irq();     
	SCB->VTOR = 0x20000000;
	
	 /*Initialize all port */
	Port_Init(); 

	/*Configure the system clock to HSI 32 MHz */
	SystemClock_Config();
		
	/* Infinite loop */
	mainloop();  

	return (0);
}


#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
