/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private function prototypes ---------------------------------------*/
void SysTick_Handler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void tt_flash_iap_pgm(void);
void tt_flash_iap_erase(void);
void GPIO_Configure(void);
void SysTick_Configure(void);
void FMC_WriteEraseSelfPGMRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"************************************************\n\r"
" WriteErase Self PGM \n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
" flash write Self PGM \n\r"
"************************************************\n\r";
uint32_t t500msec;
uint32_t tflag;

/**********************************************************************
 * @brief		SysTick_Handler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT(void)
{
	if(msec)msec--;
	
	t500msec++;
	if (t500msec>=500){
		if (tflag==0){
			tflag=1;
			HAL_GPIO_SetPin(PE,_BIT(1));
		}
		else {
			tflag=0;
			HAL_GPIO_ClearPin(PE,_BIT(1));
		}
	}
		
}

/**********************************************************************
 * @brief		tt_flash_iap_pgm
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	tt_flash_iap_pgm(void)
{
	int	i;
	unsigned char	fbuffer[512];
	
	for(i=0;i<512;i++) {
		fbuffer[i]= i&0xff;
	}
	__disable_irq();
	ProgramPage(0x3F000000,512, fbuffer);
	__enable_irq();
	
}

/**********************************************************************
 * @brief		tt_flash_iap_erase
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	tt_flash_iap_erase(void)
{
	__disable_irq();
	EraseSector(0x3F000000);		
	__enable_irq();
}

/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/* Test Pin setting PE0*/
	HAL_GPIO_ConfigOutput(PE, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 0, DISPUPD);	
	HAL_GPIO_ClearPin(PE, _BIT(0));

	/* Test Pin setting PE1*/
	HAL_GPIO_ConfigOutput(PE, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 1, DISPUPD);
	HAL_GPIO_ClearPin(PE, _BIT(1));

}

/**********************************************************************
 * @brief		SysTick_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SysTick_Configure(void)
{
	/*SysTick count setting*/
	t500msec=0;
	tflag=0;
	/*1msec interrupt*/
	SysTick_Config(SystemCoreClock/1000);   
	
}
	

/**********************************************************************
 * @brief		FMC_WriteEraseSelfPGMRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void FMC_WriteEraseSelfPGMRun(void)
{	
		msec = 500;
		while(msec);
		HAL_GPIO_ClearPin(PE,_BIT(0));
		msec = 500;
		while(msec);
		HAL_GPIO_SetPin(PE,_BIT(0));

		tt_flash_iap_erase();
		__NOP();	
		
		tt_flash_iap_pgm();
		__NOP();
		
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*SysTick Configure*/
	SysTick_Configure();

	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
   /* Infinite loop */
		FMC_WriteEraseSelfPGMRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}



#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

