/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private function prototypes ---------------------------------------*/
void GPIOAB_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void delay (void);
void GPIO_Configure(void);
void EXTI_Configure(void);
void GPIO_ExitIntRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
" GPIO demo \n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
" This example used to configure the\n\r "
" GPIO PA0,1 as source of external Interrupt \n\r"
"************************************************\n\r";

/**********************************************************************
 * @brief		GPIOAB_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void GPIOAB_IRQHandler_IT(void)
{
	uint32_t j;
	uint32_t status;

		/* GPIAB interrupt */
		status = HAL_GPIO_EXTI_GetState(PA);
		if(status & (3UL<<(0<<1)))  /*PA0*/
		{
			HAL_GPIO_EXTI_ClearPin(PA, status & (3UL<<(0<<1)));
			_DBG("\r\nPA0 Interrupt \r\n");
			for (j= 0; j<5; j++)
			{
				HAL_GPIO_SetPin(PE, _BIT(0));
				delay();
				HAL_GPIO_ClearPin(PE, _BIT(0));
				delay();
			}
		}			
	
}


/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/*********************************************************************
 * @brief		Delay function
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void delay (void) 
{
	uint32_t i;
	
	for (i = 0; i < 0x100000; i++) ;
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/* Test Pin setting PE0*/
	HAL_GPIO_ConfigOutput(PE, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 0, DISPUPD);	
	HAL_GPIO_ClearPin(PE, _BIT(0));
	
	/* Test Pin setting PE1*/
	HAL_GPIO_ConfigOutput(PE, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 1, DISPUPD);	
	HAL_GPIO_ClearPin(PE, _BIT(1));

	/* external interrupt test pin PA0*/
	HAL_GPIO_ConfigOutput(PA, 0, INPUT);
	HAL_GPIO_ConfigPullup(PA, 0, ENPU);	
	HAL_GPIO_SetDebouncePin(PA, 1);
}

/**********************************************************************
 * @brief		EXTI_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void EXTI_Configure(void)
{
	uint8_t val_level_or_edge, mode;
	
  /* external interrupt clock setting*/
	HAL_SCU_MiscClockConfig(4,PD0_TYPE,CLKSRC_LSI,100);

	val_level_or_edge = IER_LEVEL;
	mode = 0x01; //low lever interrupt or falling edge interrupt mode
	HAL_GPIO_EXTI_Config(PA, 0, val_level_or_edge, mode);

	NVIC_SetPriority(GPIOAB_IRQn, 3);	
	NVIC_EnableIRQ(GPIOAB_IRQn);
}
	

/**********************************************************************
 * @brief		GPIO_ExitIntRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_ExitIntRun(void)
{	
		delay();
		HAL_GPIO_SetPin(PE, _BIT(1));
		delay();
		HAL_GPIO_ClearPin(PE, _BIT(1));
		
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*EXTI Configure*/
	EXTI_Configure();

	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
   /* Infinite loop */
		GPIO_ExitIntRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
