/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private function prototypes ---------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void GPIO_MiscClockConfigure(void);
void GPIO_ExitIntRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
" GPIO Input demo \n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
" GPIO Input test \n\r"
"************************************************\n\r";
uint8_t pin;
PORT_Type *PCx;
uint32_t status;

/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/* Test Pin setting PE0*/
	HAL_GPIO_ConfigOutput(PE, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 0, DISPUPD);	
	HAL_GPIO_SetPin(PE, _BIT(0));
	
	/* Test Pin setting PA5*/
	PCx = PA;
	pin = 5;
	
	HAL_GPIO_ConfigOutput(PCx, pin, INPUT);
	HAL_GPIO_ConfigPullup(PCx, pin, DISPUPD);	
	HAL_GPIO_SetDebouncePin(PCx, pin);

	
}

/**********************************************************************
 * @brief		GPIO_MiscClockConfigure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_MiscClockConfigure(void)
{
	if(PCx==PA || PCx==PB || PCx==PC){
		/*LSI=500KHz, N=100, 500KHz/100=5KHz */
	  HAL_SCU_MiscClockConfig(4,PD0_TYPE,CLKSRC_LSI,100);  
	}
	else {
		HAL_SCU_MiscClockConfig(4,PD1_TYPE,CLKSRC_LSI,100);
	}
}
	

/**********************************************************************
 * @brief		GPIO_ExitIntRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_ExitIntRun(void)
{	
	   status = (HAL_GPIO_ReadPin(PCx) & (1<<pin));
		if (status == (1<<pin)){
			HAL_GPIO_ClearPin(PE, _BIT(0));
			#ifdef _DEBUG_MSG
			_DBG("\n\rPE0 readValue ");
			_DBD(status);  
			_DBG("\r\n");
			#endif
		}
		else {
			HAL_GPIO_SetPin(PE, _BIT(0));
			#ifdef _DEBUG_MSG
			_DBG("\n\rPE0 readValue ");	
			_DBD(status);  
			_DBG("\r\n");
			#endif
	
		}
		
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*EXTI Configure*/
	GPIO_MiscClockConfigure();

	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
   /* Infinite loop */
		GPIO_ExitIntRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}



#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
