/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void SysTick_Configure(void);
void GPIO_LedBlinkyRun(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"************************************************\n\r"
" GPIO demo \n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
" Using GPIO test \n\r"
"************************************************\n\r";


/**********************************************************************
 * @brief		SysTick handler sub-routine (1ms)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT(void)
{
	if(msec)msec--;
}


/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
   // Test Pin setting PE0
	HAL_GPIO_ConfigOutput(PE, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 0, DISPUPD);	
	HAL_GPIO_SetPin(PE, _BIT(0));

	// Test Pin setting PE1
	HAL_GPIO_ConfigOutput(PE, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 1, DISPUPD);
	HAL_GPIO_SetPin(PE, _BIT(1));

	// Test Pin setting PE2
	HAL_GPIO_ConfigOutput(PE, 2, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 2, DISPUPD);
	HAL_GPIO_SetPin(PE, _BIT(2));

	// Test Pin setting PE3
	HAL_GPIO_ConfigOutput(PE, 3, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 3, DISPUPD);	
	HAL_GPIO_SetPin(PE, _BIT(3));

	// Test Pin setting PE4
	HAL_GPIO_ConfigOutput(PE, 4, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 4, DISPUPD);
	HAL_GPIO_SetPin(PE, _BIT(4));

	// Test Pin setting PE5
	HAL_GPIO_ConfigOutput(PE, 5, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 5, DISPUPD);
	HAL_GPIO_SetPin(PE, _BIT(5));
	
}


/**********************************************************************
 * @brief		SysTick_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SysTick_Configure( void)
{
	/*1msec interrupt */
	SysTick_Config(SystemCoreClock/1000);   
}


/**********************************************************************
 * @brief		GPIO_LedBlinkyRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_LedBlinkyRun(void)
{	
	unsigned char i;
	
	msec = 200;
	while(msec);
	
	for(i=0;i<6;i++){
		msec = 50;
		while(msec);
		/* LED clear pin */
		HAL_GPIO_ClearPin(PE,_BIT(i));
	}
	
	for(i=0;i<6;i++){
		msec = 50;
		while(msec);
		/* LED set pin */
		HAL_GPIO_SetPin(PE,_BIT(i));
	}
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*SysTick Configure*/
	SysTick_Configure();

	 /* Enable IRQ Interrupts */
	__enable_irq();
	
   /* Infinite loop */
	while(1)
	{
		/* LED Toggle */
		GPIO_LedBlinkyRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  
	


	return (0);
}


/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
