/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private function prototypes ---------------------------------------*/
void SysTick_Handler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void Buffer_Init(uint8_t* buffer, uint8_t type);
void App_I2C_Polling_Master(void);
void App_I2C_Interrupt_Master(void);
void App_I2C_Polling_Slave (void);
void App_I2C_Interrupt_Slave (void);
void GPIO_Configure(void);
void I2C_Configure(void);
void I2C_MasterSlaveRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"\n\r************************************************\n\r"
"I2C demo \n\r"
"\t - MCU: A31G112 \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"\n\rPress 1-4 to select I2C running mode:\n\r\n\r"
"\t 1 - I2C Polling   Master\n\r"
"\t 2 - I2C Interrupt Master\n\r"
"\t 3 - I2C Polling   Slave \n\r"
"\t 4 - I2C Interrupt Slave \n\r"
"************************************************\n\r";
const uint8_t  menu1[] = "\n\r \t - Press x to exit this mode!\n\r";
const uint8_t  menu3[] = "\t - Press c to continue...\n\r";
/** Max buffer length */
#define BUFFER_SIZE			10

#define M24C02_ADDR    (0xA0>>1)
/** Own Slave address in Slave I2C device */
#define I2CDEV_S_ADDR	(0xC0>>1)

uint32_t msec;
Bool complete;
Bool isMasterMode = TRUE;
/*These global variables below used in interrupt mode - Slave device */
uint8_t Master_Buf[BUFFER_SIZE];
uint8_t dispSlave_Buf[BUFFER_SIZE];
uint8_t buffer = 0xFF;
/**********************************************************************
 * @brief		I2C0_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void I2C0_IRQHandler_IT(void)
{
	if (isMasterMode)/* Master Mode*/
	{
		HAL_I2C_Master_IRQHandler_IT(I2C0);
		if (HAL_I2C_Master_GetState(I2C0)){
			complete = TRUE;
		}
	}
	else/* Slave Mode*/
	{
		HAL_I2C_Slave_IRQHandler_IT(I2C0);
		if (HAL_I2C_Slave_GetState(I2C0))
		{
			complete = TRUE;
		}
	}	
}


/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/*********************************************************************//**
 * @brief		Initialize buffer
 * @param[in]	buffer	buffer to initialize
 * @param[in]	type:
 * 				- 0: Initialize buffer with 0
 * 				- 1: Initialize buffer with increment value from 0
 * @return 		None
 **********************************************************************/
void Buffer_Init(uint8_t* buffer, uint8_t type)
{
	uint8_t i;

	if (type) {
		for (i = 0; i < BUFFER_SIZE; i++) {
			buffer[i] = i;
		}
	}
	else	{
		for (i = 0; i < BUFFER_SIZE; i++) {
			buffer[i] = 0;
		}
	}
}


/*********************************************************************//**
 * @brief		I2C master application in polling mode
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void App_I2C_Polling_Master(void)
{
 	I2C_M_SETUP_Type transferMCfg;
	uint8_t buffer = 0xFF, regAddr = 0x00;
	uint8_t Master_Buf[BUFFER_SIZE];

	while (1) {
		#ifdef _DEBUG_MSG
		_DBG_("\n\r\n\rI2C Polling Master is running...\n\r");
		_DBG_(menu1);
		_DBG_(menu3);
		#endif

		while (1)
		{
			buffer = _DG;
			if (buffer == 'x' || buffer == 'X')
			{
				return;
			}
			else if (buffer == 'c' || buffer == 'C')
			{
				break;
			}
		}
		/* Transmit -------------------------------------------------------- */
		#ifdef _DEBUG_MSG
		_DBG_("\n\rPress '1' to transmit");
		#endif
		while (1)
		{
			buffer = _DG;
			if (buffer == '1')
				break;
		}
		#ifdef _DEBUG_MSG
		_DBG_("\n\rTransmitting...");
    #endif
		/* Initialize buffer */
		Buffer_Init(Master_Buf, 0);
		Master_Buf[0]=0x00; //address
		Master_Buf[1]=0xa2; //data	
		Master_Buf[2]=0xa4; //data	
		Master_Buf[3]=0xa6; //data	
		Master_Buf[4]=0xa8; //data		
		Master_Buf[5]=0xaA; //data		
		transferMCfg.sl_addr7bit = M24C02_ADDR;
		transferMCfg.tx_data = Master_Buf;
		transferMCfg.tx_length = 6;
		HAL_I2C_Master_Transmit(I2C0, &transferMCfg, I2C_TRANSFER_POLLING);

		/* Transmit and receive -------------------------------------------------------- */
		#ifdef _DEBUG_MSG
		_DBG_("\n\rPress '2' to Transmit, then repeat start and receive...");
		#endif
		while (1)
		{
			buffer = _DG;
			if (buffer == '2')
				break;
		}

		/* Initialize buffer */
		Buffer_Init(Master_Buf, 0);

		transferMCfg.sl_addr7bit = M24C02_ADDR;
		transferMCfg.tx_data = &regAddr;
		transferMCfg.tx_length = 1;
		transferMCfg.rx_data = Master_Buf;
		transferMCfg.rx_length = 5;
		transferMCfg.tx_count = 0;
		transferMCfg.rx_count = 0;
		HAL_I2C_MasterTransferData(I2C0, &transferMCfg, I2C_TRANSFER_POLLING);
		__NOP();
	}
}

/***********************************************************************
 * @brief		I2C master application using interrupt
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void App_I2C_Interrupt_Master(void)
{
	I2C_M_SETUP_Type transferMCfg;
	uint8_t buffer = 0xFF, regAddr = 0;
	uint8_t Master_Buf[BUFFER_SIZE];

	while (1) {
		#ifdef _DEBUG_MSG
		_DBG_("\n\r\n\rI2C Interrupt Master is running...\n\r");
		_DBG_(menu1);
		_DBG_(menu3);
		#endif

		while (1)
		{
			buffer = _DG;
			if (buffer == 'x' || buffer == 'X')
			{
				return;
			}
			else if (buffer == 'c' || buffer == 'C')
			{
				break;
			}
		}
		/* Transmit -------------------------------------------------------- */
		#ifdef _DEBUG_MSG
		_DBG_("\n\rPress '1' to transmit");
		#endif
		while (1)
		{
			buffer = _DG;
			if (buffer == '1')
				break;
		}
		#ifdef _DEBUG_MSG
		_DBG_("\n\rTransmitting...");
		#endif

		/* Initialize buffer */
		complete = FALSE;
		Buffer_Init(Master_Buf, 0);
		Master_Buf[0]=0x00; //address
		Master_Buf[1]=0x2A; //data	
		Master_Buf[2]=0x4A; //data	
		Master_Buf[3]=0x6A; //data	
		Master_Buf[4]=0x8A; //data		
		Master_Buf[5]=0xA5; //data		
		transferMCfg.sl_addr7bit = M24C02_ADDR;
		transferMCfg.tx_data = Master_Buf;
		transferMCfg.tx_length = 6;
		HAL_I2C_Master_Transmit(I2C0, &transferMCfg, I2C_TRANSFER_INTERRUPT);
		while(!complete);

		/* Transmit and receive -------------------------------------------------------- */
		#ifdef _DEBUG_MSG
		_DBG_("\n\rPress '2' to Transmit, then repeat start and receive...");
		#endif
		while (1)
		{
			buffer = _DG;
			if (buffer == '2')
				break;
		}

		/* Initialize buffer */
		complete = FALSE;
		Buffer_Init(Master_Buf, 0);
		transferMCfg.sl_addr7bit = M24C02_ADDR;
		transferMCfg.tx_data = &regAddr;
		transferMCfg.tx_length = 1;
		transferMCfg.rx_data = Master_Buf;
		transferMCfg.rx_length = 5;
		transferMCfg.tx_count = 0;
		transferMCfg.rx_count = 0;
		
		HAL_I2C_MasterTransferData(I2C0, &transferMCfg, I2C_TRANSFER_INTERRUPT);
		while(!complete);
		__NOP();
	}
}

/**********************************************************************
 * @brief		I2C slave application in polling mode
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void App_I2C_Polling_Slave (void)
{
	I2C_S_SETUP_Type transferSCfg;
	uint8_t buffer = 0xFF,i=0,j;
	uint8_t Slave_Buf[BUFFER_SIZE], S_buff[2];

	while (1) {
		#ifdef _DEBUG_MSG
		_DBG_("\n\r\n\rI2C Polling Slave is running...\n\r");
		_DBG_(menu1);
		_DBG_(menu3);
		#endif

		while (1)
		{
			buffer = _DG;
			if (buffer == 'x' || buffer == 'X')
			{
				return;
			}
			else if (buffer == 'c' || buffer == 'C')
			{
				break;
			}
		}
		/* Receive -------------------------------------------------------- */
		#ifdef _DEBUG_MSG
		_DBG_("\n\rPress '1' to Start");
		#endif

		while (1)
		{
			buffer = _DG;
			if (buffer == '1')
				break;
		}
		#ifdef _DEBUG_MSG
		_DBG_("\n\rReceiving...");
		#endif

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 0);

		transferSCfg.rx_data = Slave_Buf;
		transferSCfg.rx_length = 3;
		HAL_I2C_Slave_Receive(I2C0, &transferSCfg, I2C_TRANSFER_POLLING);
		for (j=0;j<BUFFER_SIZE;j++)
		{
			dispSlave_Buf[j]=Slave_Buf[j];
		}
		__NOP();

		/* Receive and transmit -------------------------------------------------------- */
		#ifdef _DEBUG_MSG
		_DBG_("\n\rStart Receive, wait for repeat start and transmit...");
		#endif

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 1);

		transferSCfg.rx_data = S_buff;
		transferSCfg.rx_length = 2;
		Slave_Buf[0]=((i++)&0xff);//0x8A; //data	
		Slave_Buf[1]=0xAA; //data		
		Slave_Buf[2]=0xA5; //data	
		transferSCfg.tx_data = Slave_Buf;
		transferSCfg.tx_length = 3;
		HAL_I2C_SlaveTransferData(I2C0, &transferSCfg, I2C_TRANSFER_POLLING);
		__NOP();		
	}
}

/*********************************************************************//**
 * @brief		I2C slave application using interrupt
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void App_I2C_Interrupt_Slave (void)
{
	I2C_S_SETUP_Type transferSCfg;
	uint8_t buffer = 0xFF;
	uint8_t Slave_Buf[BUFFER_SIZE], S_buff[2];

	while (1) {
		#ifdef _DEBUG_MSG
		_DBG_("\n\r\n\rI2C Interrupt Slave is running...\n\r");
		_DBG_(menu1);
		_DBG_(menu3);
		#endif

		while (1)
		{
			buffer = _DG;
			if (buffer == 'x' || buffer == 'X')
			{
				return;
			}
			else if (buffer == 'c' || buffer == 'C')
			{
				break;
			}
		}
		/* Receive -------------------------------------------------------- */
		#ifdef _DEBUG_MSG
		_DBG_("\n\rPress '1' to Start");
		#endif

		while (1)
		{
			buffer = _DG;
			if (buffer == '1')
				break;
		}
		#ifdef _DEBUG_MSG
		_DBG_("\n\rReceiving...");
		#endif

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 0);

		complete = FALSE;
		transferSCfg.rx_data = Slave_Buf;
		transferSCfg.rx_length = 3;
		HAL_I2C_Slave_Receive(I2C0, &transferSCfg, I2C_TRANSFER_INTERRUPT);
		 while(!complete);
		__NOP();
		
		/* Receive and transmit -------------------------------------------------------- */
		#ifdef _DEBUG_MSG
		_DBG_("\n\rStart Receive, wait for repeat start and transmit...");
		#endif

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 0);

		complete = FALSE;
		
		transferSCfg.rx_data = S_buff;
		transferSCfg.rx_length = 2;
		Slave_Buf[0]=0x6A; //data	
		Slave_Buf[1]=0x8A; //data		
		Slave_Buf[2]=0xA5; //data		
		transferSCfg.tx_data = Slave_Buf;
		transferSCfg.tx_length = 3;
		HAL_I2C_SlaveTransferData(I2C0, &transferSCfg, I2C_TRANSFER_INTERRUPT);
		 while(!complete);
		__NOP();
	}
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{

	/* Test Pin setting PE0*/
	HAL_GPIO_ConfigOutput(PE, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 0, DISPUPD);	
	HAL_GPIO_ClearPin(PE, _BIT(0));
	
	
	/* I2C0 PF6:SCL0, PF7:SDA0*/
	HAL_GPIO_ConfigOutput((PORT_Type*)PF, 6, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction((PORT_Type*)PF, 6, FUNC2);
	HAL_GPIO_ConfigOutput((PORT_Type*)PF, 7, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction((PORT_Type*)PF, 7, FUNC2);
	//PCU_ConfigureDebounce((PORT_Type*)PF, 6, ENABLE);
	//PCU_ConfigureDebounce((PORT_Type*)PF, 7, ENABLE);
}

/**********************************************************************
 * @brief		I2C_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void I2C_Configure(void)
{
	 /*Initialize Slave I2C peripheral*/
	if(HAL_I2C_Init(I2C0, 100000) != HAL_OK)
	{
		 /* Initialization Error */
     Error_Handler();
	}
	/* Set  Own slave address for I2C device */
	HAL_I2C_Slave_SetAddress0(I2C0, I2CDEV_S_ADDR, DISABLE);
	HAL_I2C_Slave_SetAddress1(I2C0, I2CDEV_S_ADDR, DISABLE);
	
}
	

/**********************************************************************
 * @brief		I2C_MasterSlaveRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void I2C_MasterSlaveRun(void)
{	
	buffer = _DG;

	switch (buffer)
	{
	case '1': //Master Polling Mode
		App_I2C_Polling_Master();
		DEBUG_MenuPrint();
		break;
	case '2': //Master Interrupt Mode
		isMasterMode = TRUE;
		App_I2C_Interrupt_Master();
		DEBUG_MenuPrint();
		break;
	case '3': //Slave Polling Mode
		App_I2C_Polling_Slave();
		DEBUG_MenuPrint();
		break;
	case '4': //Slave Interrupt Mode
		isMasterMode = FALSE;
		App_I2C_Interrupt_Slave();
		DEBUG_MenuPrint();
		break;
	default:
		break;
	}
		
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*ADC Configure*/
	I2C_Configure();

	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
               /* Infinite loop */
		I2C_MasterSlaveRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
