/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.01
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define SEG_MAX_NUM (10)    // Max SEG port Number
#define COM_MAX_NUM (16)    // Max COM port Number
#define FND_MAX_NUM (17)    // Max FND Type Number (Character types)
#define FNDPOS_MAX_NUM (15)    // Max FND Type Number (Character types)

#define SEG0        (0x00000001)
#define SEG1        (0x00000002)
#define SEG2        (0x00000004)
#define SEG3        (0x00000008)
#define SEG4        (0x00000010)
#define SEG5        (0x00000020)
#define SEG6        (0x00000040)
#define SEG7        (0x00000080)
#define SEG8        (0x00000100)
#define SEG9        (0x00000200)

#define COM0        (0x00000001)
#define COM1        (0x00000002)
#define COM2        (0x00000004)
#define COM3        (0x00000008)
#define COM4        (0x00000010)
#define COM5        (0x00000020)
#define COM6        (0x00000040)
#define COM7        (0x00000080)
#define COM8        (0x00000100)
#define COM9        (0x00000200)
#define COM10       (0x00000400)
#define COM11       (0x00000800)
#define COM12       (0x00001000)
#define COM13       (0x00002000)
#define COM14       (0x00004000)
#define COM15       (0x00008000)

#define DISPRAM0    *(volatile unsigned long *)0x40006044
#define DISPRAM1    *(volatile unsigned long *)0x40006048
#define DISPRAM2    *(volatile unsigned long *)0x4000604C
#define DISPRAM3    *(volatile unsigned long *)0x40006050
#define DISPRAM4    *(volatile unsigned long *)0x40006054
#define DISPRAM5    *(volatile unsigned long *)0x40006058
#define DISPRAM6    *(volatile unsigned long *)0x4000605C
#define DISPRAM7    *(volatile unsigned long *)0x40006060
#define DISPRAM8    *(volatile unsigned long *)0x40006064
#define DISPRAM9    *(volatile unsigned long *)0x40006068
#define DISPRAM10    *(volatile unsigned long *)0x4000606C
#define DISPRAM11    *(volatile unsigned long *)0x40006070
#define DISPRAM12    *(volatile unsigned long *)0x40006074
#define DISPRAM13    *(volatile unsigned long *)0x40006078
#define DISPRAM14    *(volatile unsigned long *)0x4000607C
#define DISPRAM15    *(volatile unsigned long *)0x40006080

#define DIMM_HIGH   (0xDAUL)

#define FND_OUT(X, Y)   LED->SEGOE=(FND[X].fnd_val); LED->COMOE=(FND_POS[Y].fnd_val);

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT(void);
void LED_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void LED_Configure(void);
void SysTick_Configure(void);
void LED_BlinkyRun(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
uint32_t disp_sec;
const uint8_t menu[] =
"************************************************\n\r"
" LED Driver demo \n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
" Using LED Driver test \n\r"
"************************************************\n\r";

typedef struct {
    uint8_t     seg_id;
    PORT_Type*  port_group;
    uint8_t     port_num;
    char*       port_name;
} SEG_PORT_TYPE;

typedef struct {
    uint8_t     com_id;
    PORT_Type*  port_group;
    uint8_t     port_num;
    char*       port_name;
} COM_PORT_TYPE;

typedef struct {
    uint8_t     fnd_id;
    uint32_t    fnd_val;
} FND_TYPE;

SEG_PORT_TYPE SEG[SEG_MAX_NUM] = {
    {0, PE, 2, "SEG0"},
    {1, PE, 3, "SEG1"},
    {2, PE, 4, "SEG2"},
    {3, PE, 5, "SEG3"},
    {4, PE, 6, "SEG4"},
    {5, PE, 7, "SEG5"},
    {6, PD, 5, "SEG6"},
    {7, PD, 4, "SEG7"},
    {8, PD, 3, "SEG8"},
    {9, PD, 2, "SEG9"},
};

COM_PORT_TYPE COM[COM_MAX_NUM] = {
    { 0, PD, 1, "COM0"},
    { 1, PD, 0, "COM1"},
    { 2, PC, 4, "COM2"},
    { 3, PC, 3, "COM3"},
    { 4, PC, 2, "COM4"},
    { 5, PC, 1, "COM5"},
    { 6, PC, 0, "COM6"},
    { 7, PB, 2, "COM7"},
    { 8, PB, 1, "COM8"},
    { 9, PB, 0, "COM9"},
    {10, PA, 7, "COM10"},
    {11, PA, 6, "COM11"},
    {12, PA, 5, "COM12"},
    {13, PA, 4, "COM13"},
    {14, PA, 3, "COM14"},
    {15, PA, 2, "COM15"},
};

FND_TYPE    FND[FND_MAX_NUM] = {
    {0x0, (SEG0 | SEG1 | SEG2 | SEG3 | SEG4 | SEG5)},
    {0x1, (SEG1 | SEG2)},
    {0x2, (SEG0 | SEG1 | SEG3 | SEG4 | SEG6)},
    {0x3, (SEG0 | SEG1 | SEG2 | SEG3 | SEG6)},
    {0x4, (SEG1 | SEG2 | SEG5 | SEG6)},
    {0x5, (SEG0 | SEG2 | SEG3 | SEG5 | SEG6)},
    {0x6, (SEG0 | SEG2 | SEG3 | SEG4 | SEG5 | SEG6)},
    {0x7, (SEG0 | SEG1 | SEG2)},
    {0x8, (SEG0 | SEG1 | SEG2 | SEG3 | SEG4 | SEG5 | SEG6)},
    {0x9, (SEG0 | SEG1 | SEG2 | SEG3 | SEG5 | SEG6 )},
    {0xA, (SEG0 | SEG1 | SEG2 | SEG4 | SEG5 | SEG6 )},
    {0xB, (SEG2 | SEG3 | SEG4 | SEG5 | SEG6 )},
    {0xC, (SEG3 | SEG4 | SEG6 )},
    {0xD, (SEG1 | SEG2 | SEG3 | SEG4 | SEG6 )},
    {0xE, (SEG0 | SEG3 | SEG4 | SEG5 | SEG6 )},
    {0xF, (SEG0 | SEG4 | SEG5 | SEG6 )},
    {0x10, (SEG7)},
};

FND_TYPE    FND_POS[FNDPOS_MAX_NUM] = {
    {0x0, COM0 },
    {0x1, COM1 },
    {0x2, COM2 },
    {0x3, COM3 },
    {0x4, COM4 },
    {0x5, COM5 },
    {0x6, COM6 },
    {0x7, COM7 },
    {0x8, COM9 },
    {0x9, COM10},
    {0xA, COM11},
    {0xB, COM12},
    {0xC, COM13},
    {0xD, COM14},
    {0xE, COM15},
};

/**********************************************************************
 * @brief		SysTick handler sub-routine (1ms)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT(void)
{
	    while(msec) {
        msec--;
    }
}


/**********************************************************************
 * @brief		LED_IRQHandler
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void LED_IRQHandler_IT(void) 
{
    uint32_t Status;
    
    Status = LED->SR;

    if(Status & LED_SR_LED_INT_Msk) {       // Check LED Interrupt Flag
        LED->SR &= ~(LED_SR_LED_INT_Msk);   // Clear LED Interrupt Flag
    }
}

/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		seg_init
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void seg_init(SEG_PORT_TYPE seg) 
{
	HAL_GPIO_ConfigOutput(seg.port_group, seg.port_num, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(seg.port_group, seg.port_num, FUNC4);
  HAL_GPIO_ConfigPullup(seg.port_group, seg.port_num, DISABLE);
}

/**********************************************************************
 * @brief		com_init
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void com_init(COM_PORT_TYPE com) 
{
	HAL_GPIO_ConfigOutput(com.port_group, com.port_num, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(com.port_group, com.port_num, FUNC4);
  HAL_GPIO_ConfigPullup(com.port_group, com.port_num, DISABLE);
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void) 
{
    int i;

    for (i = 0 ; i < SEG_MAX_NUM; i++) {
        seg_init(SEG[i]);   	// Init All SEG ports (GPIO)
    }

    for (i = 0 ; i < COM_MAX_NUM; i++) {
        com_init(COM[i]);   	// Init All COM ports (GPIO)
    }
}

/**********************************************************************
 * @brief		LED_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void LED_Configure(void) 
{
    /* LED Peripheral Init */
    SCU->PER2 &= ~SCU_PER2_LED_Msk;
    SCU->PER2 |= SCU_PER2_LED_Msk;

    /* LED Peripheral Clock Init */
    SCU->PCER2 &= ~SCU_PCER2_LED_Msk;
    SCU->PCER2 |= SCU_PCER2_LED_Msk;

    /* LED Peripheral Clock Setting */
    HAL_SCU_MiscClockConfig(5, LED_TYPE, CLKSRC_HSI, 1);   // LED Clock = 32MHz/1 = 32MHz

    /*COM Output Enable Register*/
    LED->COMOE = 0 //0: Normal I/O, 1: COM enable
        | (0x01<<15)              // COM15
        | (0x01<<14)              // COM14
        | (0x01<<13)              // COM13
        | (0x01<<12)              // COM12
        | (0x01<<11)              // COM11
        | (0x01<<10)              // COM10
        | (0x01<<9)               // COM9
        | (0x01<<8)               // COM8
        | (0x01<<7)               // COM7
        | (0x01<<6)               // COM6
        | (0x01<<5)               // COM5
        | (0x01<<4)               // COM4
        | (0x01<<3)               // COM3
        | (0x01<<2)               // COM2
        | (0x01<<1)               // COM1
        | (0x01<<0)               // COM0
        ;
    
    /*SEG Output Enable Register*/
    LED->SEGOE = 0 //0: Normal I/O, 1: SEG enable
        | (0x01<<9)               // SEG9
        | (0x01<<8)               // SEG8
        | (0x01<<7)               // SEG7
        | (0x01<<6)               // SEG6
        | (0x01<<5)               // SEG5
        | (0x01<<4)               // SEG4
        | (0x01<<3)               // SEG3
        | (0x01<<2)               // SEG2
        | (0x01<<1)               // SEG1
        | (0x01<<0)               // SEG0
        ;
    
    /*LED Prescaler data Register*/
     LED->PRESD = 0x80; // 32MHz / 128 = 250kHz

    /*COM Enable Register*/
    LED->COMER = 0
        | (0x01<<15)              // PC3    COM15
        | (0x01<<14)              // PC4    COM14
        | (0x01<<13)              // PD0    COM13
        | (0x01<<12)              // PD1    COM12
        | (0x01<<11)              // PD2    COM11
        | (0x01<<10)              // PD3    COM10
        | (0x01<<9)               // PD4    COM9
        | (0x01<<8)               // PD5    COM8
        | (0x01<<7)               // PE7    COM7
        | (0x01<<6)               // PE6    COM6
        | (0x01<<5)               // PE5    COM5
        | (0x01<<4)               // PE4    COM4
        | (0x01<<3)               // PE3    COM3
        | (0x01<<2)               // PE2    COM2
        | (0x01<<1)               // PE1    COM1
        | (0x01<<0)               // PE0    COM0
        ;
    
    /*COM Pulse Width Control Register*/
    LED->COMPWID = (0xFF << LED_COMPWID_COMPWID_Pos);  // 8bit

    /*COM Dimming Control Register*/
    //Dimmed COM Width = COMPWID - Overlaptiome-LEDCLK*COMDIMMn
    LED->DIMM1 = 0 // each 8bit
        | (DIMM_HIGH << LED_DIMM1_COMDIMM3_Pos)              // COM3
        | (DIMM_HIGH << LED_DIMM1_COMDIMM2_Pos)              // COM2
        | (DIMM_HIGH << LED_DIMM1_COMDIMM1_Pos)              // COM1
        | (DIMM_HIGH << LED_DIMM1_COMDIMM0_Pos)              // COM0
        ;
    LED->DIMM2 = 0 // each 8bit
        | (DIMM_HIGH << LED_DIMM2_COMDIMM7_Pos)              // COM7
        | (DIMM_HIGH << LED_DIMM2_COMDIMM6_Pos)              // COM6
        | (DIMM_HIGH << LED_DIMM2_COMDIMM5_Pos)              // COM5
        | (DIMM_HIGH << LED_DIMM2_COMDIMM4_Pos)              // COM4
        ;
    LED->DIMM3 = 0 // each 8bit
        | (DIMM_HIGH << LED_DIMM3_COMDIMM11_Pos)             // COM11
        | (DIMM_HIGH << LED_DIMM3_COMDIMM10_Pos)             // COM10
        | (DIMM_HIGH << LED_DIMM3_COMDIMM9_Pos)              // COM9
        | (DIMM_HIGH << LED_DIMM3_COMDIMM8_Pos)              // COM8
        ;
    LED->DIMM4 = 0 // each 8bit
        | (DIMM_HIGH << LED_DIMM4_COMDIMM15_Pos)             // COM15
        | (DIMM_HIGH << LED_DIMM4_COMDIMM14_Pos)             // COM14
        | (DIMM_HIGH << LED_DIMM4_COMDIMM13_Pos)             // COM13
        | (DIMM_HIGH << LED_DIMM4_COMDIMM12_Pos)             // COM12
        ;

    /*LED STOP Duration Register*/
    //LED STOP Duration Width = fCLK/(LEDSTPD+1)
    LED->STPD = ((0x3A8C0*2) << LED_PRESD_PRESD_Pos); // 10ms 19bit

    /*Init LED Display RAM Register DISPRAM0 to DISPRAM15 */
    DISPRAM0 = 0x00000000;
    DISPRAM1 = 0x00000000;    
    DISPRAM2 = 0x00000000;    
    DISPRAM3 = 0x00000000;    
    DISPRAM4 = 0x00000000;
    DISPRAM5 = 0x00000000;
    DISPRAM6 = 0x00000000;
    DISPRAM7 = 0x00000000;
    DISPRAM8 = 0x00000000;
    DISPRAM9 = 0x00000000;
    DISPRAM10 = 0x00000000;
    DISPRAM11 = 0x00000000;
    DISPRAM12 = 0x00000000;
    DISPRAM13 = 0x00000000;  
    DISPRAM14 = 0x00000000;
    DISPRAM15 = 0x00000000;

    LED->CON2 = (1 << LED_CON2_OVERLAP_Pos) | (5 << LED_CON2_OVERTS_Pos);    // OVERLAP TIME = NONE = 16MHz/32 = 2us

    LED->SR = (1 << LED_SR_LED_INTE_Pos);      // LED Interrupt Enable

    // Interrupt Enable
	  NVIC_SetPriority(LED_IRQn, 3);
	  NVIC_EnableIRQ(LED_IRQn); // Enable Interrupt for TIMERn channel

    LED->CON1 = (0 << LED_CON1_MD_Pos) | (1 << LED_CON1_LEDEN_Pos) | (0 << LED_CON1_LEDST_Pos);   // LED Alone Mode, LED Enable, Stop LED Operation
}

/**********************************************************************
 * @brief		LED_BlinkyRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void LED_BlinkyRun(void) 
{
    int i;
    
    /*LED Display RAM Register DISPRAM0 to DISPRAM15 */
    DISPRAM0 = 0x000003FF;
    DISPRAM1 = 0x000003FF;    
    DISPRAM2 = 0x000003FF;    
    DISPRAM3 = 0x000003FF;    
    DISPRAM4 = 0x000003FF;
    DISPRAM5 = 0x000003FF;
    DISPRAM6 = 0x000003FF;
    DISPRAM7 = 0x000003FF;
    DISPRAM8 = 0x000003FF;
    DISPRAM9 = 0x000003FF;
    DISPRAM10 = 0x000003FF;
    DISPRAM11 = 0x000003FF;
    DISPRAM12 = 0x000003FF;
    DISPRAM13 = 0x000003FF;  
    DISPRAM14 = 0x000003FF;
    DISPRAM15 = 0x000003FF;        
    
    for(i = 0 ; i < 5; i++) {        
        FND_OUT(i, i+2);    
        LED->CON1 |= (1 << LED_CON1_LEDST_Pos);

        msec = 1;
        while(msec);
    }
}



/**********************************************************************
 * @brief		SysTick_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SysTick_Configure( void)
{
	/*1msec interrupt */
	SysTick_Config(SystemCoreClock/1000);   
}



/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*Configure LED peripheral*/
	LED_Configure();
	
	/*SysTick Configure*/
	SysTick_Configure();

	 /* Enable IRQ Interrupts */
	__enable_irq();
	
   /* Infinite loop */
	while(1)
	{
		/* LED Toggle */
		LED_BlinkyRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  
	


	return (0);
}


/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
