/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private function prototypes ---------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void LVI_Configure(void);
void LVI_PollingRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"************************************************\n\r"
"LVI demo\n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
"************************************************\n\r";
LVI_CFG_Type	LVI_config;



/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/* Test Pin setting PE0*/
	HAL_GPIO_ConfigOutput(PE, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 0, DISPUPD);	
	HAL_GPIO_ClearPin(PE, _BIT(0));

}

/**********************************************************************
 * @brief		ADC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void LVI_Configure(void)
{
	LVI_config.EnSel = ENABLE;
	LVI_config.LvlSel = LVI_LVL_3_20V;
	
	if(HAL_LVI_Init(&LVI_config)  != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();
	}
	
}
	

/**********************************************************************
 * @brief		LVI_PollingRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void LVI_PollingRun(void)
{	
	if (HAL_SCU_LVI_GetLviFlag()){
			HAL_SCU_LVI_ClrLviFlag();
			HAL_GPIO_SetPin(PE, _BIT(0));
		}
		else {
			HAL_GPIO_ClearPin(PE, _BIT(0));			
		}
		
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*LVI Configure*/
	LVI_Configure();

	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
		LVI_PollingRun();
   /* Infinite loop */
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
