/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private function prototypes ---------------------------------------*/
void GPIOCD_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void EXIT_Configure(void);
void delay(void); 
void PWR_SleepRun(void);
void PWR_SetPinLoop(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"************************************************\n\r"
"Sleep demo\n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
"************************************************\n\r";

/**********************************************************************
 * @brief		SysTick handler sub-routine (1ms)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT (void) 					
{
	/* SysTick Interrupt Handler @ 1000Hz*/
	if(msec)msec--;
}


/**********************************************************************
 * @brief		GPIOE Handler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void GPIOE_IRQHandler_IT(void)
{
	uint32_t  status;
	status = HAL_GPIO_EXTI_GetState(PE);
	if (status & (3<<(7<<1)))
	{
		HAL_GPIO_EXTI_ClearPin(PE, status&(3<<(7<<1)));
	}
}


/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/*********************************************************************//**
 * @brief		delay
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void delay(void) 
{
	uint32_t i;
	
	for (i = 0; i < 0x1000 ; i++) ;
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/* Test Pin setting PE0 */
	HAL_GPIO_ConfigOutput(PE, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 0, DISPUPD);	
	HAL_GPIO_ClearPin(PE, _BIT(0));
	

	/* External Interrupt Test pin PE6*/
	HAL_GPIO_ConfigOutput(PE, 1, INPUT);
	HAL_GPIO_ConfigPullup(PE, 1, ENPU);
	HAL_GPIO_ClearPin(PE, _BIT(1));
  //HAL_GPIO_SetDebouncePin(PE,6);
	
	/* External Test pin PE7*/
	HAL_GPIO_ConfigOutput(PE, 7, INPUT);
	HAL_GPIO_ConfigPullup(PE, 7, ENPU);
	HAL_GPIO_ClearPin(PE, _BIT(7));
	//HAL_GPIO_SetDebouncePin(PE,7);
	
}

/**********************************************************************
 * @brief		EXIT_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void EXIT_Configure(void)
{
	
  HAL_GPIO_EXTI_Config(PE, 7, IER_EDGE, 1);
	HAL_SCU_LVRCmd(DISABLE);
	
	NVIC_SetPriority(GPIOE_IRQn, 3);
	NVIC_EnableIRQ(GPIOE_IRQn);
	
	HAL_SCU_WakeUpSRCCmd(WAKEUP_GPIOE, ENABLE);
		
}


/**********************************************************************
 * @brief		PWR_DeepSleepRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void PWR_SleepRun(void)
{
	#ifdef _DEBUG_MSG
  _DBG("Press PE1 to Enter Sleep Mode !");
	#endif
	
	while(HAL_GPIO_ReadPin(PE) & (1UL<<1));
	
	#ifdef _DEBUG_MSG
	_DBG("\n\rEnter Sleep... Connect PE7 pin to GND to exit sleep mode");
	#endif
	
	HAL_PWR_EnterSleepMode();
	
	#ifdef _DEBUG_MSG
	_DBG("\n\rWaked Up from Sleep");
	#endif
}

/**********************************************************************
 * @brief		PWR_SetPinLoop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void PWR_SetPinLoop(void)
{	
	
		HAL_GPIO_SetPin(PE, _BIT(0));
		delay();
	
		HAL_GPIO_ClearPin(PE, _BIT(0));
		delay();
		
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*EXIT Configure*/
	EXIT_Configure();
  	
	/* Enable IRQ Interrupts */
	__enable_irq();
	
	/*sleep Run*/
	PWR_SleepRun();

	while(1)
	{
		PWR_SetPinLoop();
   /* Infinite loop */
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
