/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.01
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private function prototypes ---------------------------------------*/
void SPI20_IRQHandler_IT(void);
void SPI21_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void SPI20_Configure(void);
void SPI21_Configure(void);
void SPI20_MasterRun(void);
void SPI21_SlaveRun(void);
void SPI_MasterSlaveRunloop(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/
uint32_t msec;          
const uint8_t cmdm[] =
"A31G213 start 1 > ";

const uint8_t menu[] =
"\n\r"
"************************************************\n\r"
" A31G21x_TEST_EXAMPLE \n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
"\t SPI INTERRUPT MODE TEST \n\r"
"\t MASTER : SPI1, SLAVE : SPI0 \n\r"
"\t 1: SPI0 SPI1 Data \n\r"
"************************************************\n\r";

uint32_t fflag;
volatile uint32_t rxdata, txdata;
SPI_CFG_Type SP_config;




/**********************************************************************
 * @brief		SPI20_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SPI20_IRQHandler_IT(void)
{
	uint32_t status;
	
	status = HAL_SPI_GetStatus(SPI20);
	if ( (status & SPI_STAT_TXBUF_EMPTY) && (HAL_SPI_GetControl(SPI20) & SPI_INTCFG_TXIE)){ // tx buffer empty
		HAL_SPI_ConfigInterrupt(SPI20, SPI_INTCFG_TXIE, DISABLE);
		
		HAL_SPI_Enable(SPI20, DISABLE);  // SPI disable 	
		HAL_SPI_TransmitData(SPI20, txdata);	
		HAL_SPI_Enable(SPI20, ENABLE);  // SPI enable	
		
	}
}

/**********************************************************************
 * @brief		SPI21_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SPI21_IRQHandler_IT(void)
{
	uint32_t status;
	
	status = HAL_SPI_GetStatus(SPI21);
	
	if ( (status & SPI_STAT_RXBUF_READY) && (HAL_SPI_GetControl(SPI21) & SPI_INTCFG_RXIE)){ // rx buffer full		
		rxdata = HAL_SPI_ReceiveData(SPI21);
		_DBG(" rx = "); _DBH(rxdata);	_DBG("\n\r");
		fflag=1;
	} 
}

/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	_DBG(cmdm);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/* SPI20*/	
	// MOSI20 (PC2)	 
	HAL_GPIO_ConfigOutput(PC, 2, ALTERN_FUNC);    
	HAL_GPIO_ConfigFunction(PC, 2, FUNC2);
	
	// MISO20 (PC3)	
	HAL_GPIO_ConfigOutput(PC, 3, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PC, 3, FUNC2);
	
	//SCK20 (PC4)   
	HAL_GPIO_ConfigOutput(PC, 4, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PC, 4, FUNC2);
	
	//SS20 (PD0) : slave set as SSx 
	HAL_GPIO_ConfigOutput(PD, 0, ALTERN_FUNC);
	//PCU_SetDirection(PD, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigFunction(PD, 0, FUNC2);


	/* SPI21*/
	// MOSI10 (PE5)	
	HAL_GPIO_ConfigOutput(PE, 5, ALTERN_FUNC);   
	HAL_GPIO_ConfigFunction(PE, 5, FUNC2);
	
	// MISO10 (PE4)	
	HAL_GPIO_ConfigOutput(PE, 4, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PE, 4, FUNC2);
	
	//SCK10 (PE3)   
	HAL_GPIO_ConfigOutput(PE, 3, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PE, 3, FUNC2);
	
	//SS10 (PE2)   : master control as GPIO 
	HAL_GPIO_ConfigOutput(PE, 2, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PE, 2, FUNC2);

	
}

/**********************************************************************
 * @brief		SPI20_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SPI20_Configure(void)
{
  //SPIx_IRQn = SPI20_IRQn;
						
			/* SPI enable */
	SP_config.Databit = SPI_DS_8BITS;
	SP_config.CPHA = SPI_CPHA_LO;
	SP_config.CPOL = SPI_CPOL_LO;
	SP_config.DataDir = SPI_MSB_FIRST;
	SP_config.Mode = SPI_MASTER_MODE;
	SP_config.BaudRate = 0xff; // PCLK / (79+1) 
	
  /*Initialize Slave I2C peripheral*/
	if(HAL_SPI_Init(SPI20, &SP_config)!= HAL_OK)
	{
	    /* Initialization Error */
     Error_Handler();
	}

}
	

/**********************************************************************
 * @brief		SPI21_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SPI21_Configure(void)
{
	//SPIx_IRQn = SPI21_IRQn;
				
	/* SPI enable */
	SP_config.Databit = SPI_DS_8BITS;
	SP_config.CPHA = SPI_CPHA_LO;
	SP_config.CPOL = SPI_CPOL_LO;
	SP_config.DataDir = SPI_MSB_FIRST;
	SP_config.Mode = SPI_SLAVE_MODE;
	SP_config.BaudRate = 0xff; // PCLK / (79+1) 
	
   /*Initialize Slave I2C peripheral*/
	if(HAL_SPI_Init(SPI21, &SP_config)!= HAL_OK)
	{
	    /* Initialization Error */
     Error_Handler();
	}

}

/**********************************************************************
 * @brief		SPI20_Run
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SPI20_MasterRun(void)
{	
    //SPI_IntConfig(SPI20, SPI_INTCFG_RXIE, ENABLE); // rx interrupt enable
		HAL_SPI_Enable(SPI20, ENABLE);  // SPI enable 

		/* preemption = 1, sub-priority = 1 */
		NVIC_SetPriority(SPI20_IRQn, ((0x01<<1)|0x01));
		NVIC_EnableIRQ(SPI20_IRQn);
}


/**********************************************************************
 * @brief		SPI1_Run
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SPI21_SlaveRun(void)
{	
		HAL_SPI_ConfigInterrupt(SPI21, SPI_INTCFG_RXIE, ENABLE); // rx interrupt enable 
		HAL_SPI_Enable(SPI21, ENABLE);  // SPI enable 

		/* preemption = 1, sub-priority = 2 */
		NVIC_SetPriority(SPI21_IRQn, ((0x01<<1)|0x02));
		NVIC_EnableIRQ(SPI21_IRQn);
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void SPI_MasterSlaveRunloop(void)
{
	uint32_t i, j;
	uint8_t		 ch_rtn;
	
	ch_rtn = _DG;
		
	_DBC(ch_rtn);
	_DBG("\r\n");
	
	if (ch_rtn == 0x31){
	i = 0x40;	
		
	while(1){
		txdata = i;	
		
	  #ifdef _DEBUG_MSG 
		_DBG(" tx = "); _DBH(txdata);
	  #endif
		rxdata = 0;
		fflag=0;
		HAL_SPI_ConfigInterrupt(SPI20, SPI_INTCFG_TXIE, ENABLE);

		while(fflag == 0);					// check rx buffer full 
							
		i++;
		i&=(0xff);

		for(j=0;j<100000;j++);

		if (i==0x49)
		{
			break;
		}
	}
}
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
  /*SPI20 Configure*/
	SPI20_Configure();
	
	/*SPI21 Configure*/
	SPI21_Configure();
	
	/*SPI21 Master run*/
	SPI20_MasterRun();
	
	/*SPI21 slave run*/
	SPI21_SlaveRun();

	/* Enable IRQ Interrupts */
	__enable_irq();

   /* Infinite loop */
	SPI_MasterSlaveRunloop();

}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
