/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private function prototypes ---------------------------------------*/
void TIMER20_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void TIMER2n_Configure(void);
void TIMER2n_MatchInterruptRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"************************************************\n\r"
"Timer Match interrupt demo \n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
" Using Timer 2n in interrupt mode\n\r"
"************************************************\n\r";
TIMER2n_PERIODICCFG_Type TIMER2n_Config;
__IO	uint32_t count=0;
__IO	uint32_t flag =0;

/**********************************************************************
 * @brief		TIMER20_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TIMER20_IRQHandler_IT(void)
{
	if (HAL_TIMER2n_GetMatchInterrupt(TIMER20) == 1){
		HAL_TIMER2n_ClearMatchInterrupt(TIMER20);
		count++;
		if (count >= 200){
			count = 0;
			if (flag==0){
				flag=1;
				HAL_GPIO_ClearPin(PE, _BIT(0));					
			}
			else {
				flag=0;
				HAL_GPIO_SetPin(PE, _BIT(0));
			}	
		}	
	}

}
/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/* Test Pin setting PE0*/
	HAL_GPIO_ConfigOutput(PE, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 0, DISPUPD);	
	HAL_GPIO_SetPin(PE, _BIT(0));
  
	/* T20C pin config set if need*/
	HAL_GPIO_ConfigOutput(PC, 0, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PC, 0, FUNC1);

	
}

/**********************************************************************
 * @brief		TIMER2n_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER2n_Configure(void)
{	
  /*Timer2n clock source from PCLK*/
  HAL_SCU_Timer20_ClockConfig(TIMER20CLK_PCLK);
	TIMER2n_Config.CkSel = TIMER2n_MCCR2PCLK;    
	TIMER2n_Config.Prescaler = 32;    /* 32Mhz / 32 = 1Mhz ->1us*/
	TIMER2n_Config.ADR = (1000); 
	TIMER2n_Config.StartLevel = TIMER2n_START_LOW;   
 
	if(HAL_TIMER2n_Init(TIMER20, TIMER2n_PERIODIC_MODE, &TIMER2n_Config) != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();
	}
	HAL_TIMER2n_ConfigInterrupt(TIMER20, TIMER2n_CR_MATCH_INTR, ENABLE);
	
	/* Enable Interrupt for TIMERx channel */
	NVIC_SetPriority(TIMER20_IRQn, 3);
	NVIC_EnableIRQ(TIMER20_IRQn); 
		
}
	

/**********************************************************************
 * @brief		TIMER2n_MatchInterruptRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER2n_MatchInterruptRun(void)
{	
	/*  timer start*/
	HAL_TIMER2n_Cmd(TIMER20, ENABLE);
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*TIMER2n Configure*/
	TIMER2n_Configure();

	/*InterruptRun*/
	TIMER2n_MatchInterruptRun();
	
	/* Enable IRQ Interrupts */
	__enable_irq();
	

	while(1)
	{
   /* Infinite loop */

	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
