/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/

#define PWM_HZ			6000  /* pwm */

/* Private function prototypes ---------------------------------------*/
void TIMER20_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void TIMER2n_Configure(void);
void TIMER2n_PWMRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"Timer PWM demo \n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
" Using Timer 30 in PWM mode\n\r"
"************************************************\n\r";
__IO	uint32_t count;
__IO	uint32_t count1;
__IO	uint32_t flag;
__IO	uint32_t flag1;

/**********************************************************************
 * @brief		TIMER30_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TIMER30_IRQHandler_IT(void)
{
	if ((HAL_TIMER3n_GetStatus_IT(TIMER30) & TIMER30INT_PMATEN) == TIMER30INT_PMATEN)
	{
		if (flag==0){
			flag=1;
			HAL_GPIO_ClearPin(PC, _BIT(0));					
		}
		else {
			flag=0;
			HAL_GPIO_SetPin(PC, _BIT(0));
		}	
	}
	
	HAL_TIMER3n_ClearStatus_IT(TIMER30,0x7F);	
}


/**********************************************************************
 * @brief		ADC_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void ADC_IRQHandler_IT(void)
{
	uint16_t status;
	status = HAL_ADC_GetStatus(ADC);

	if (status & ADC_STAT_END){
		
		if (flag1==0){
			flag1=1;
			HAL_GPIO_ClearPin(PC, _BIT(1));					
		}
		else {
			flag1=0;
			HAL_GPIO_SetPin(PC, _BIT(1));
		}	
		
		HAL_ADC_ClearStatus(ADC);	
	}
}



/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
  /* Test Pin setting PC0*/
	HAL_GPIO_ConfigOutput(PC, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PC, 0, DISPUPD);	
	HAL_GPIO_ClearPin(PC, _BIT(0));
	
	/* Test Pin setting PC1*/
	HAL_GPIO_ConfigOutput(PC, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PC, 0, DISPUPD);	
	HAL_GPIO_ClearPin(PC, _BIT(1));
	
	/* PWM30AA(PE0)*/
	HAL_GPIO_ConfigOutput(PE, 0, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PE, 0, FUNC1);
	/* PWM30AB(PE1)*/
	HAL_GPIO_ConfigOutput(PE, 1, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PE, 1, FUNC1);
	/* PWM30BA(PE2)*/
	HAL_GPIO_ConfigOutput(PE, 2, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PE, 2, FUNC1);
	/* PWM30BB(PE3)*/
	HAL_GPIO_ConfigOutput(PE, 3, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PE, 3, FUNC1);
	/* PWM30CA(PE4)*/
	HAL_GPIO_ConfigOutput(PE, 4, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PE, 4, FUNC1);
	/* PWM30CB(PE5)*/
	HAL_GPIO_ConfigOutput(PE, 5, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PE, 5, FUNC1);
		
}


/**********************************************************************
 * @brief		TIMER3n_Configure
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TIMER3n_Configure(void)
{
	
	/*Timer30 clock source from PCLK*/
	HAL_SCU_Timer30_ClockConfig(TIMER30CLK_PCLK); 
	
	if(HAL_TIMER3n_Init(TIMER30) != HAL_OK)
	{
		Error_Handler();
	}
	
	HAL_TIMER3n_SetPeriod(TIMER30,0);		/* 32MHz/(0+1) = 32MHz*/
	
	HAL_TIMER3n_SetPeriod(TIMER30,(uint16_t)(((uint32_t)40000000L / (PWM_HZ)) / 2 -1)); //8000Hz = 125us
	HAL_TIMER3n_SetADuty(TIMER30,(TIMER30->PDR>>1)); 
	HAL_TIMER3n_SetBDuty(TIMER30,(TIMER30->PDR>>1));
	HAL_TIMER3n_SetCDuty(TIMER30,(TIMER30->PDR>>1));
		
	HAL_TIMER3n_OutputCtrl(TIMER30,ENABLE,TIMER30O_APOLOW,TIMER30O_BPOHIGH);
	
	HAL_TIMER3n_ConfigInterrupt(TIMER30,ENABLE,TIMER30INT_PMATEN);
	HAL_TIMER3n_ClearStatus_IT(TIMER30,0x7F);
	
	HAL_TIMER3n_SetADCTrigger(TIMER30,TIMER30ADT_BTTGEN,0);
	
	HAL_TIMER3n_MPWMCmd(TIMER30,TIMER30_UPMATCH,TIMER30_E1PERIOD);
	
	HAL_TIMER3n_SetDelayTime(TIMER30,TIMER30_DLYINSEN,TIMER30_INSFRONT, 63);	// (63 + 1) / 32MHz = 2us
	

	NVIC_EnableIRQ(TIMER30_IRQn);
	NVIC_SetPriority(TIMER30_IRQn, 3);
}

/**********************************************************************
 * @brief		ADC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_Configure(void)
{
	ADC_CFG_Type AD_config;
	
	AD_config.RefSel = ADC_REF_VDD;   
	AD_config.TrgSel = ADC_TRIGGER_TIMER30;	
	AD_config.InClkDiv = 12;      // 32M/17 =2.7M 
	if(HAL_ADC_Init(ADC, &AD_config) != HAL_OK)
	{
		Error_Handler();
	}

	HAL_ADC_ConfigInterrupt(ADC, ENABLE);

	NVIC_SetPriority(ADC_IRQn, 3);
	NVIC_EnableIRQ(ADC_IRQn);	

}
	

/**********************************************************************
 * @brief		TIMER3n_PWMRun
 * @param[in]	None
 * @return	None
 **********************************************************************/
void TIMER3n_PWMRun(void)
{
	HAL_TIMER3n_Start(TIMER30,TIMER30_ENABLE);	
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*TIMER3n Configure*/
	TIMER3n_Configure();
 
	/*ADC Configure*/
	ADC_Configure();
	
	/*timer30 start*/
	TIMER3n_PWMRun();
	
	/* Enable IRQ Interrupts */
	__enable_irq();
	

	while(1)
	{
   /* Infinite loop */
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
