/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/
//USART10 <-> USART11
#define USART10_M_USART11_S	1
#define USART10_S_USART11_M	0

/* Private function prototypes ---------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void USART_SPI_SS_control(USART_Type *USART1n, char ch);
void GPIO_Configure(void);
void SPI_USART10Configure(void);
void SPI_USART11Configure(void);
void SPI_PollingRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"************************************************\n\r"
" GPIO demo \n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
" Using SPI Polling test \n\r"
"************************************************\n\r";
uint8_t tx_ch=0, rx_ch=0;
uint32_t tmp;
USART_CFG_Type USARTConfigStruct;



/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		USART_SPI_SS_control
 * @param[in]	None
 * @return		None
 **********************************************************************/
void USART_SPI_SS_control(USART_Type *USART1n, char ch)
{
	if (USART1n == USART10){
		if (ch==0) 
			PB->OUTDR &= ~(1<<3);
		else 
			PB->OUTDR |= (1<<3);
	}
	else if (USART1n == USART11){
		if (ch==0) 
			PD->OUTDR &= ~(1<<5);
		else 
			PD->OUTDR |= (1<<5);		
	}

}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/*
	 * Initialize USART10
	 */
	/* MOSI10 (PB0)	*/
	HAL_GPIO_ConfigOutput(PB, 0, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 0, FUNC2);
	/* MISO10 (PB1)	*/
	HAL_GPIO_ConfigOutput(PB, 1, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 1, FUNC2);
 	/*SCK10 (PB2) */  
    HAL_GPIO_ConfigOutput(PB, 2, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 2, FUNC2);
	#if USART10_M_USART11_S
	/*SS10 (PB3)   : master control as GPIO */
	HAL_GPIO_ConfigOutput(PB, 3, PUSH_PULL_OUTPUT);
	#endif
	#if USART10_S_USART11_M
	/*SS10 (PB3)   : slave set as SSx */
	HAL_GPIO_ConfigOutput(PB, 3, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 3, FUNC2);
	#endif
	
	/*
	 * Initialize USART11
	 */
	// MOSI11 (PD2)	 
	HAL_GPIO_ConfigOutput(PD, 2, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PD, 2, FUNC2);
	// MISO11 (PD3)	
	HAL_GPIO_ConfigOutput(PD, 3, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PD, 3, FUNC2);
	//SCK11 (PD4) 
	HAL_GPIO_ConfigOutput(PD, 4, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PD, 4, FUNC2);
	#if USART10_M_USART11_S
	//SS11 (PD5)   : slave set as SSx 
	HAL_GPIO_ConfigOutput(PD, 5, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PD, 5, FUNC2);
	#endif
	#if USART10_S_USART11_M
	//SS11 (PD5)   : master control as GPIO 
	HAL_GPIO_ConfigOutput(PD, 5, PUSH_PULL_OUTPUT);
	#endif
}

/**********************************************************************
 * @brief		SPI_USART10Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SPI_USART10Configure(void)
{
  HAL_USART_SPI_Mode_Config(&USARTConfigStruct);
	
	if(HAL_USART_Init(USART10, &USARTConfigStruct) != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();
	}
	#if USART10_M_USART11_S
	// SPI Master
	HAL_USART_DataControlConfig(USART10, USART_CONTROL_MASTER, ENABLE);
	USART_SPI_SS_control(USART10, 1); // SS init
	#endif
	#if USART10_S_USART11_M
	// SPI slave
	HAL_USART_DataControlConfig(USART10, USART_CONTROL_MASTER, DISABLE);
	// Slave SS Function Enable
	HAL_USART_DataControlConfig(USART10, USART_CONTROL_USTSSEN, ENABLE);
	#endif
	
}

/**********************************************************************
 * @brief		SPI_USART11Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SPI_USART11Configure(void)
{
	/* USART11 initialize*/
	if(HAL_USART_Init(USART11, &USARTConfigStruct)!= HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();
	}

	#if USART10_M_USART11_S
	// SPI slave
	HAL_USART_DataControlConfig(USART11, USART_CONTROL_MASTER, DISABLE);
	// Slave SS Function Enable
	HAL_USART_DataControlConfig(USART11, USART_CONTROL_USTSSEN, ENABLE);
	#endif
	#if USART10_S_USART11_M
	// SPI Master
	HAL_USART_DataControlConfig(USART11, USART_CONTROL_MASTER, ENABLE);
	USART_SPI_SS_control(USART11, 1); // SS init
	#endif

	
}

/**********************************************************************
 * @brief		SPI_PollingRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SPI_PollingRun(void)
{
	#ifdef _DEBUG_MSG
	_DBG("MASTER SEND DATA ");
	tx_ch = _DG;
	_DBG(" = "); _DBC(tx_ch);_DBG(" : ");_DBH(tx_ch); _DBG_(" ");

	rx_ch = tx_ch +1;
	_DBG("SLAVE SEND DATA = "); _DBC(rx_ch);_DBG(" = ");_DBH(rx_ch); _DBG_(" ");
  #endif
	
	#if USART10_M_USART11_S
	USART11->DR = rx_ch; 
	USART_SPI_SS_control(USART10, 0);
	HAL_USART_Transmit(USART10, &tx_ch,1);
	while(!(HAL_USART_GetStatus(USART10)&0x40)){} // wait 
	tx_ch = USART10->DR;
	USART10->ST = 0x40;
	USART_SPI_SS_control(USART10, 1);
	HAL_USART_Receive(USART11,&rx_ch,1);
	#endif

	#if USART10_S_USART11_M
	USART10->DR = rx_ch; 
	USART_SPI_SS_control(USART11, 0);
	HAL_USART_Transmit(USART11, &tx_ch,1);
	while(!(HAL_USART_GetStatus(USART11)&0x40)){} // wait 
	tx_ch = USART11->DR;
	USART11->ST = 0x40;
	USART_SPI_SS_control(USART11, 1);
	HAL_USART_Receive(USART10,&rx_ch,1);
	#endif

	#ifdef _DEBUG_MSG
	_DBG("MASTER GET DATA = "); _DBC(tx_ch);_DBG(" = ");_DBH(tx_ch); _DBG_(" ");
	_DBG("SLAVE GET DATA = "); _DBC(rx_ch);_DBG(" = ");_DBH(rx_ch); _DBG_(" ");_DBG_(" ");
	#endif

}




/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*SPI USART10 Configure*/
	SPI_USART10Configure();
	
	/*SPI USART11 Configure*/
	SPI_USART11Configure();

	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
   /* Infinite loop */
		SPI_PollingRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
