/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/
//USART10 <-> USART11
#define USART10_Tx_USART11_Rx	0
#define USART10_Rx_USART11_Tx	1

/* Private function prototypes ---------------------------------------*/
void USART10_IRQHandler_IT(void);
void USART11_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void USARTSend(USART_Type *USARTx, uint8_t *txbuf, uint8_t buflen);
void USARTReceive(USART_Type *USARTx, uint8_t *rxbuf, uint8_t buflen);
void GPIO_Configure(void);
void UART_USART10Configure(void);
void UART_USART11Configure(void);
void UART_InterruptRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"************************************************\n\r"
" UART demo \n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
" USART Uart Interrupt test \n\r"
"************************************************\n\r";
__IO FlagStatus TxIntStat;// Current Tx Interrupt enable state
uint8_t tx_ch='a', rx_ch;
uint8_t _usart10_rx_buffer[255], _usart10_buffer_tail, _usart10_buffer_head;
uint8_t _usart11_rx_buffer[255], _usart11_buffer_tail, _usart11_buffer_head;
uint8_t _usart12_rx_buffer[255], _usart12_buffer_tail, _usart12_buffer_head;
uint8_t _usart13_rx_buffer[255], _usart13_buffer_tail, _usart13_buffer_head;

uint8_t _usart10_tx_flag, _usart10_rx_flag;
uint8_t _usart11_tx_flag, _usart11_rx_flag;
uint8_t _usart12_tx_flag, _usart12_rx_flag;
uint8_t _usart13_tx_flag, _usart13_rx_flag;

USART_CFG_Type USARTConfigStruct;	
/*********************************************************************
 * @brief		USART10 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART10_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp;

	/* Determine the interrupt source */
	intsrc = HAL_USART_GetStatus(USART10);
	tmp = intsrc & USART_SR_BITMASK;

	// Receive Data Available or Character time-out
	if ((tmp & USART_SR_RXC) == USART_SR_RXC){
		_usart10_rx_buffer[_usart10_buffer_tail++] = HAL_USART_ReceiveByte(USART10);
		_usart10_rx_flag = 1;
	}

	// Transmit Holding Empty
	if ((tmp & USART_SR_TXC) == USART_SR_TXC){
		HAL_USART_ClearStatus(USART10, USART_STATUS_TXC);
		_usart10_tx_flag = 1;
	}
}

/*********************************************************************
 * @brief		USART11 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART11_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp;

	/* Determine the interrupt source */
	intsrc = HAL_USART_GetStatus(USART11);
	tmp = intsrc & USART_SR_BITMASK;

	// Receive Data Available or Character time-out
	if ((tmp & USART_SR_RXC) == USART_SR_RXC){
    _usart11_rx_buffer[_usart11_buffer_tail++] = HAL_USART_ReceiveByte(USART11);
		_usart11_rx_flag = 1;
	}

	// Transmit Holding Empty
	if ((tmp & USART_SR_TXC) == USART_SR_TXC){
     HAL_USART_ClearStatus(USART11, USART_STATUS_TXC);
		_usart11_tx_flag = 1;
	}
}


/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		USARTSend
 * @param[in]	None
 * @return		None
 **********************************************************************/
void USARTSend(USART_Type *USARTx, uint8_t *txbuf, uint8_t buflen)
{
    uint32_t i, timeout;
    uint8_t *tx_flag;
    
    if(USARTx == USART10){
        tx_flag = &_usart10_tx_flag;
    }else if(USARTx == USART11){
        tx_flag = &_usart11_tx_flag;
		
    }
        
    for(i=0;i<buflen;i++){
        timeout = 10000;
        HAL_USART_TransmitByte(USARTx, (txbuf[i]));
        while(timeout--){
            if(*tx_flag) break;
        }
		*tx_flag = 0;
    }
}

/**********************************************************************
 * @brief		USARTReceive
 * @param[in]	None
 * @return		None
 **********************************************************************/
void USARTReceive(USART_Type *USARTx, uint8_t *rxbuf, uint8_t buflen)
{
    uint32_t tmp, i;
    uint8_t *buf, *head, *tail, *flag;
    if(USARTx == USART10){
        buf = _usart10_rx_buffer;
        head = &_usart10_buffer_head;
        tail = &_usart10_buffer_tail;
				flag = &_usart10_rx_flag;
    }else if(USARTx == USART11){
        buf = _usart11_rx_buffer;
        head = &_usart11_buffer_head;
        tail = &_usart11_buffer_tail;
				flag = &_usart11_rx_flag;
    
    }
	while(*flag != 1){}
    *flag = 0;
	
    if(*head > *tail){
        tmp = *tail+256;
    }else{
        tmp = *tail;
    }
    
    for(i=0;(i<buflen && *head<tmp);i++, (*head)++){
        rxbuf[i] = buf[*head];
    }
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/*
	* Initialize USART10 pin connect
	*/
	HAL_GPIO_ConfigOutput(PB, 1, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 1, FUNC1);
	HAL_GPIO_ConfigPullup(PB, 1, ENPU);

	HAL_GPIO_ConfigOutput(PB, 0, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 0, FUNC1);

	/*
	* Initialize USART11 pin connect
	*/
	HAL_GPIO_ConfigOutput(PD, 3, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PD, 3, FUNC1);
	HAL_GPIO_ConfigPullup(PD, 3, ENPU);

	HAL_GPIO_ConfigOutput(PD, 2, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PD, 2, FUNC1);

}

/**********************************************************************
 * @brief		UART_USART10Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_USART10Configure(void)
{
	/* default: 38400-8-N-1	*/
	HAL_USART_UART_Mode_Config(&USARTConfigStruct);	

	if(HAL_USART_Init(USART10, &USARTConfigStruct) != HAL_OK)
	{
		/* Initialization Error */
		Error_Handler();
	}
	/* Enable USART interrupt */
	HAL_USART_ConfigInterrupt((USART_Type *)USART10, USART_INTCFG_RXC, ENABLE);
	HAL_USART_ConfigInterrupt((USART_Type *)USART10, USART_INTCFG_TXC, ENABLE);

	NVIC_SetPriority(USART10_IRQn, 3);
	NVIC_EnableIRQ(USART10_IRQn);

}

/**********************************************************************
 * @brief		UART_USART11Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_USART11Configure(void)
{
	/* USART11 initialize*/
	if(HAL_USART_Init(USART11, &USARTConfigStruct)!= HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();
	}

	/* Enable USART interrupt */
	HAL_USART_ConfigInterrupt((USART_Type *)USART11, USART_INTCFG_RXC, ENABLE);
	HAL_USART_ConfigInterrupt((USART_Type *)USART11, USART_INTCFG_TXC, ENABLE);

	NVIC_SetPriority(USART11_IRQn, 3);
	NVIC_EnableIRQ(USART11_IRQn);
	
}

/**********************************************************************
 * @brief		UART_InterruptRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_InterruptRun(void)
{
	#ifdef _DEBUG_MSG
	_DBG("Tx DATA ");
	tx_ch = _DG;
	_DBG(" = "); _DBC(tx_ch);_DBG(" : ");_DBH(tx_ch); _DBG_(" ");
	#endif

	#if USART10_Tx_USART11_Rx   //USART10 Tx , USART11 Rx
	USARTSend(USART10, &tx_ch, 1);
	USARTReceive(USART11, &rx_ch, 1);
	#endif

	#if USART10_Rx_USART11_Tx   //USART11 Tx , USART10 Rx
	USARTSend(USART11, &tx_ch, 1);
	USARTReceive(USART10, &rx_ch, 1);
	#endif


	#ifdef _DEBUG_MSG
	_DBG("Rx DATA = "); _DBC(rx_ch);_DBG(" = ");_DBH(rx_ch); _DBG_(" ");_DBG_(" ");
	#endif
}




/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*UART USART10 Configure*/
	UART_USART10Configure();
	
	/*UART USART11 Configure*/
	UART_USART11Configure();

	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
   /* Infinite loop */
		UART_InterruptRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
