/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/
//USART10 <-> USART11
#define USART10_M_USART11_S	1
#define USART10_S_USART11_M	0

/* Private function prototypes ---------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void USART_USART10Configure(void);
void USART_USART11Configure(void);
void USART_PollingRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"************************************************\n\r"
" USART demo \n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
" A31G21x Usrt Polling test \n\r"
"************************************************\n\r";
uint8_t tx_ch='a', rx_ch;

USART_CFG_Type USARTConfigStruct;	


/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/*
	* Initialize USART10 pin connect
	*/
	HAL_GPIO_ConfigOutput(PB, 0, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 0, FUNC1);

	HAL_GPIO_ConfigOutput(PB, 1, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 1, FUNC1);
	HAL_GPIO_ConfigPullup(PB, 1, ENPU);

	HAL_GPIO_ConfigOutput(PB, 2, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 2, FUNC2);

	/*
	* Initialize USART11 pin connect
	*/
	// TXD (PD2)	 
	HAL_GPIO_ConfigOutput(PD, 2, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PD, 2, FUNC1);
	// RXD (PD3)	
	HAL_GPIO_ConfigOutput(PD, 3, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PD, 3, FUNC1);
	HAL_GPIO_ConfigPullup(PD, 3, ENPU);
	// SCK (PD4)	
	HAL_GPIO_ConfigOutput(PD, 4, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PD, 4, FUNC2);


}

/**********************************************************************
 * @brief		USART_USART10Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void USART_USART10Configure(void)
{
	/* default: 38400-8-N-1	*/
	HAL_USART_UART_Mode_Config(&USARTConfigStruct);	

	if(HAL_USART_Init(USART10, &USARTConfigStruct) != HAL_OK)
	{
		/* Initialization Error */
		Error_Handler();
	}
  #if USART10_M_USART11_S
	// USRT Marster
	HAL_USART_DataControlConfig(USART10, USART_CONTROL_MASTER, ENABLE);
	HAL_USART_DataControlConfig(USART10, USART_CONTROL_DISSCK, ENABLE);
	#endif
	#if USART10_S_USART11_M
	// USRT Slave
	HAL_USART_DataControlConfig(USART10, USART_CONTROL_MASTER, DISABLE);
	#endif


}

/**********************************************************************
 * @brief		USART_USART11Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void USART_USART11Configure(void)
{
	/* USART11 initialize*/
	if(HAL_USART_Init(USART11, &USARTConfigStruct)!= HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();
	}
  #if USART10_M_USART11_S
	// USRT Slave
	HAL_USART_DataControlConfig(USART11, USART_CONTROL_MASTER, DISABLE);
	#endif
	#if USART10_S_USART11_M
	// USRT Marster
	HAL_USART_DataControlConfig(USART11, USART_CONTROL_MASTER, ENABLE);
	HAL_USART_DataControlConfig(USART11, USART_CONTROL_DISSCK, ENABLE);
	#endif

	
}

/**********************************************************************
 * @brief		USART_PollingRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void USART_PollingRun(void)
{
		#ifdef _DEBUG_MSG
		_DBG("MASTER SEND DATA ");
		tx_ch = _DG;
		_DBG(" = "); _DBC(tx_ch);_DBG(" : ");_DBH(tx_ch); _DBG_(" ");
   
		rx_ch = tx_ch+1;
		_DBG("SLAVE SEND DATA = "); _DBC(rx_ch);_DBG(" = ");_DBH(rx_ch); _DBG_(" ");   
    #endif	
		//USRT Communication Start
		#if USART10_M_USART11_S
		USART11->DR = rx_ch; 
		HAL_USART_Transmit(USART10, &tx_ch,1);
		while(!(HAL_USART_GetStatus(USART10) & USART_SR_TXC)){} // wait 
		tx_ch = USART10->DR;
		HAL_USART_ClearStatus(USART10, USART_STATUS_TXC);
		HAL_USART_Receive(USART11,&rx_ch,1);
		#endif
		#if USART10_S_USART11_M
		USART10->DR = rx_ch; 
		HAL_USART_Transmit(USART11, &tx_ch,1);
		while(!(HAL_USART_GetStatus(USART11) & USART_SR_TXC)){} // wait 
		tx_ch = USART11->DR;
		HAL_USART_ClearStatus(USART11, USART_STATUS_TXC);
		HAL_USART_Receive(USART10,&rx_ch,1);
		#endif
		#if USART12_M_USART13_S
		USART13->DR = rx_ch; 
		HAL_USART_Transmit(USART12, &tx_ch,1);
		while(!(HAL_USART_GetStatus(USART12) & USART_SR_TXC)){} // wait 
		tx_ch = USART12->DR;
		HAL_USART_ClearStatus(USART12, USART_STATUS_TXC);
		HAL_USART_Receive(USART13,&rx_ch,1);
		#endif
		#if USART12_S_USART13_M
		USART12->DR = rx_ch; 
		HAL_USART_Transmit(USART13, &tx_ch,1);
		while(!(HAL_USART_GetStatus(USART13) & USART_SR_TXC)){} // wait 
		tx_ch = USART13->DR;
		HAL_USART_ClearStatus(USART13, USART_STATUS_TXC);
		HAL_USART_Receive(USART12,&rx_ch,1);
		#endif
    #ifdef _DEBUG_MSG
		_DBG("MASTER GET DATA = "); _DBC(tx_ch);_DBG(" = ");_DBH(tx_ch); _DBG_(" ");
		_DBG("SLAVE GET DATA = "); _DBC(rx_ch);_DBG(" = ");_DBH(rx_ch); _DBG_(" ");_DBG_(" ");
	  #endif

}




/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*USART USART10 Configure*/
	USART_USART11Configure();
	
	/*USART USART11 Configure*/
	USART_USART11Configure();

	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
   /* Infinite loop */
		USART_PollingRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
