/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private function prototypes ---------------------------------------*/
void SysTick_Handler_IT(void);
void WDT_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void WDT_Configure(void);
void SysTick_Configure(void);
void WDT_ResetRun(void);
void WDT_ReloadTimeRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"************************************************\n\r"
"Watch dog timer reset demo\n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
"************************************************\n\r";
uint8_t wdtf_uf;  /* underflow flag */
uint8_t wdtf_wmf; /* window match flag */ 
WDT_CFG_Type wdtCfg;

/**********************************************************************
 * @brief		SysTick handler sub-routine (1ms)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT (void) 
{
	if(msec)msec--;
}


/**********************************************************************
 * @brief		WDT_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void WDT_IRQHandler_IT(void)
{
	uint32_t status;
	
	
	status = HAL_WDT_GetStatus();
	if (status & 0x01){ // Match Interrupt
		if (wdtf_uf == 0){
			wdtf_uf=1;
			HAL_GPIO_ClearPin(PE, _BIT(0));
		}
		else {
			wdtf_uf=0;
			HAL_GPIO_SetPin(PE, _BIT(0));
		}
		HAL_WDT_ClearStatus(1);	
	}

	if (status & 0x02){ // Window Interrupt
		if (wdtf_wmf == 0){
			wdtf_wmf=1;
			HAL_GPIO_ClearPin(PE, _BIT(1));
		}
		else {
			wdtf_wmf=0;
			HAL_GPIO_SetPin(PE, _BIT(1));
			HAL_WDT_ClearStatus(2);	
		}
		
	}	
}

/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
  // XIN Pin setting PF0
	HAL_GPIO_ConfigOutput(PF, 0, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PF, 0, FUNC3);
	HAL_GPIO_ConfigPullup(PF, 0, DISPUPD);
	
	// XOUT Pin setting PF1
	HAL_GPIO_ConfigOutput(PF, 1, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PF, 1, FUNC3);
	HAL_GPIO_ConfigPullup(PF, 1, DISPUPD);
	
	//SOSC
	// SXIN Pin setting PF2
	HAL_GPIO_ConfigOutput(PF, 2, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PF, 2, FUNC3);
	HAL_GPIO_ConfigPullup(PF, 2, DISPUPD);
	
	// SXOUT Pin setting PF1
	HAL_GPIO_ConfigOutput(PF, 3, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PF, 3, FUNC3);
	HAL_GPIO_ConfigPullup(PF, 3, DISPUPD);
	
	// Test Pin setting PE0
	HAL_GPIO_ConfigOutput(PE, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 0, DISPUPD);	
	HAL_GPIO_ClearPin(PE, _BIT(0));

	// Test Pin setting PE1
	HAL_GPIO_ConfigOutput(PE, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 1, DISPUPD);
	HAL_GPIO_ClearPin(PE, _BIT(1));
}

/**********************************************************************
 * @brief		ADC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void WDT_Configure(void)
{
  /*WDT clock source from WDTRC. WDTRC must set LSI clock enable!!*/
	HAL_SCU_LSI_ClockConfig(LSIOSC_EN);
	HAL_SCU_WDT_ClockConfig(WDTCLK_WDTRC);	// 31250 hz	

	HAL_SCU_SetResetSrc(RST_WDTRST, ENABLE);
	HAL_SCU_ClearResetStatus(0xff); // clear all reset status 

  /*	WDTDR(0.5s) < WDTWDR(1s), clear in 900ms */
	wdtCfg.wdtResetEn = ENABLE;
	wdtCfg.wdtClkDiv = WDT_DIV_4;  	
	wdtCfg.wdtTmrConst = 7812/2; 		// 0.5s    @31250
	wdtCfg.wdtWTmrConst = 7812; 		// 1s	 
 
	if(HAL_WDT_Init(wdtCfg)!= HAL_OK)
  {
		/* Initialization Error */
    Error_Handler();
	}
	
	HAL_WDT_ConfigInterrupt(WDT_INTCFG_UNFIEN, ENABLE);
	HAL_WDT_ConfigInterrupt(WDT_INTCFG_WINMIEN, ENABLE);	
	
  NVIC_SetPriority(WDT_IRQn, 3);
  NVIC_EnableIRQ(WDT_IRQn);
	

}
	
/**********************************************************************
 * @brief		SysTick_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SysTick_Configure(void)
{	
	/*1msec interrupt */
   SysTick_Config(SystemCoreClock/1000);  
}

/**********************************************************************
 * @brief		WDT_ResetRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void WDT_ResetRun(void)
{	
   HAL_WDT_Start(ENABLE);
}


/**********************************************************************
 * @brief		WDT_ReloadTimeRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void WDT_ReloadTimeRun(void)
{
	msec = 900; 
	while(msec);
	HAL_WDT_ReloadTimeCounter();
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*WDT Configure*/
	WDT_Configure();
  
	/*SysTick Configure*/
	SysTick_Configure();
	
	/* Enable IRQ Interrupts */
	__enable_irq();
	
	/*WDT Reset Start*/
	WDT_ResetRun();

	while(1)
	{
   /* Infinite loop */
		WDT_ReloadTimeRun();
		
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
