/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private function prototypes ---------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void WT_Configure(void);
void WT_InterruptRun(void);
void WT_SetPinRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"************************************************\n\r"
"WT polling demo\n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
" Using WT test \n\r"
"************************************************\n\r";
uint32_t status;
uint8_t wtif;  // flag
WT_CFG_Type wtCfg;





/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/* Test Pin setting PE0*/
	HAL_GPIO_ConfigOutput(PE, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 0, DISPUPD);	
	HAL_GPIO_ClearPin(PE, _BIT(0));
}

/**********************************************************************
 * @brief		WT_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void WT_Configure(void)
{
 /*WT clock source from WDTRC. WDTRC must set LSI clock enable!!*/
	HAL_SCU_LSI_ClockConfig(LSIOSC_EN);
	HAL_SCU_WT_ClockConfig(WTCLK_WDTRC);	
	wtCfg.wtClkDiv = WT_DIV_2_14_MUL_WTDR;  // WT_DIV_2_7, WT_DIV_2_13, WT_DIV_2_14, WT_DIV_2_14_MUL_WTDR	
									    // 31250Hz / ((2^14) *(1+WTDR))	
	wtCfg.wtTmrConst = 1; // WTDR Value	

 
	if(HAL_WT_Init(wtCfg)!= HAL_OK)
  {
		/* Initialization Error */
    Error_Handler();
	}
	
	

}
	

/**********************************************************************
 * @brief		WT_PollingRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void WT_PollingRun(void)
{	
   HAL_WT_Start(ENABLE);
}


/**********************************************************************
 * @brief		WT_SetPinRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void WT_SetPinRun(void)
{
	do {
			status = ( HAL_WT_GetStatus() & 0x02);
		} while(status==0);
		
		if (status & 0x02){
			if (wtif == 0){
				wtif=1;
				HAL_GPIO_ClearPin(PE, _BIT(0));
			}
			else {
				wtif=0;
				HAL_GPIO_SetPin(PE, _BIT(0));
			}
			HAL_WT_ClearStatus();
		}	
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*WT Configure*/
	WT_Configure();
  
	/*WT Interrupt Start*/
	WT_PollingRun();
	
	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
		WT_SetPinRun();
   /* Infinite loop */
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
